Package: CSAMA10
Type: Package
Title: Computational Statistics for Genome Biology 2010
Version: 0.0.3
Author: Martin Morgan <mtmorgan@fhcrc.org>, Patrick Aboyoun
    <paboyoun@fhcrc.org>, Simon Anders, Robert Gentleman, Florian Hahne,
    Wolfgang Huber, Audrey Kauffman, Gregoire Pau
Maintainer: Martin Morgan <mtmorgan@fhcrc.org>
License: Artistic-2.0
LazyLoad: yes
Description: This package contains material for 'CSAMA10', which took place
    in Bressanone-Brixen, Italy from June 13th-18th 2010.
Depends: IRanges, GenomicRanges, Biostrings, BSgenome, Rsamtools,
    ShortRead, rtracklayer, biomaRt, GenomicFeatures, EatonEtAlChIPseq,
    chipseq
Imports: IRanges, GenomicRanges, Biostrings, ShortRead, GenomicFeatures
Suggests: BSgenome.Scerevisiae.UCSC.sacCer2, org.Sc.sgd.db, KEGG.db,
    BSgenome.Hsapiens.UCSC.hg19, SNPlocs.Hsapiens.dbSNP.20090506, DESeq,
    hgu95av2probe, bitops
Packaged: 2010-07-02 23:02:31 UTC; patrick
